#ifndef _IDSA_H_
#define _IDSA_H_

/* maximum values */
#define IDSA_M_NAME        28	/* longest unit name */
#define IDSA_M_MEDIUM     128	/* for intermediate entities */
#define IDSA_M_LONG      1024	/* for long things (comments, etc) */
#define IDSA_M_MESSAGE   4096	/* size of event message */

#include <sys/types.h>

/* type definitions - crude attempt at aligning things */
#define idsa_make(i,s) ((i<<12)|(0xff8&((s&0x7)?(0x8|s):s)))

#define IDSA_T_NULL    idsa_make(0,0)	/* no type */
#define IDSA_T_STRING  idsa_make(1,IDSA_M_MEDIUM)	/* 128 char string */
#define IDSA_T_INT     idsa_make(2,sizeof(int))		/* generic integer */
#define IDSA_T_UID     idsa_make(3,sizeof(uid_t))	/* uid */
#define IDSA_T_GID     idsa_make(4,sizeof(gid_t))	/* gid */
#define IDSA_T_PID     idsa_make(5,sizeof(pid_t))	/* pid */
#define IDSA_T_TIME    idsa_make(6,sizeof(time_t))	/* time */
#define IDSA_T_FLAG    idsa_make(7,sizeof(int))		/* boolean */
#define IDSA_T_RISK    idsa_make(8,2*sizeof(short))	/* risk range */
#define IDSA_T_ERRNO   idsa_make(9,sizeof(int))		/* unix errno */
#define IDSA_T_HOST    idsa_make(10,IDSA_M_MEDIUM)	/* 128 char net/host name */
#define IDSA_T_ADDR    idsa_make(11,2*sizeof(long))	/* IPv4 + mask */
#define IDSA_T_PORT    idsa_make(12,2*sizeof(int))	/* protocol/service */
#define IDSA_T_FILE    idsa_make(13,IDSA_M_MEDIUM)	/* file (128 chars) */
#define IDSA_M_TYPES            14	/* largest type + 1 */

/* structures you should not look into ************************************* */

#ifdef __cplusplus
extern "C" {
#endif

  struct idsa_unit;
  typedef struct idsa_unit IDSA_UNIT;

  struct idsa_event;
  typedef struct idsa_event IDSA_EVENT;

  struct idsa_connection;
  typedef struct idsa_connection IDSA_CONNECTION;

/* session setup *********************************************************** */

#define IDSA_F_FAILOPEN 0x0001	/* always allow if other side broken */
#define IDSA_F_KEEP     0x0002	/* don't delete IDSA_EVENT within idsa_log */
#define IDSA_F_ENV      0x0004	/* get location of socket from environment */
#define IDSA_F_SIGPIPE  0x0008	/* do not do a signal(SIGPIPE, SIG_IGN) */
  IDSA_CONNECTION *idsa_open(char *name, char *credential, int flags);
  int idsa_close(IDSA_CONNECTION * c);

/* event setup ************************************************************* */

  IDSA_EVENT *idsa_event(IDSA_CONNECTION * c);	/* get event */
  void idsa_template(IDSA_CONNECTION * c, IDSA_EVENT * e);	/* use e as template */
  int idsa_scale(IDSA_CONNECTION * c, double ar, double cr, double ir);		/* scale risks by this value */
  void idsa_free(IDSA_CONNECTION * c, IDSA_EVENT * e);	/* needs only be called if F_KEEP set */

/* one line usage ********************************************************** */

  int idsa_scan(IDSA_CONNECTION * c, char *n, char *s, int f, double ar, double cr, double ir,...);
  int idsa_set(IDSA_CONNECTION * c, char *n, char *s, int f, double ar, double cr, double ir,...);

/* fill in details ********************************************************* */

/* units which should normally be set */
  int idsa_name(IDSA_EVENT * e, char *n);
  int idsa_scheme(IDSA_EVENT * e, char *n);
  int idsa_service(IDSA_EVENT * e, char *n);
  int idsa_risks(IDSA_EVENT * e, int f, double a, double c, double i);
  int idsa_honour(IDSA_EVENT * e, int f);
#define idsa_honor idsa_honour

/* usually no point in modifying these units */
  int idsa_pid(IDSA_EVENT * e, pid_t p);
  int idsa_uid(IDSA_EVENT * e, uid_t u);
  int idsa_gid(IDSA_EVENT * e, gid_t g);
  int idsa_time(IDSA_EVENT * e, time_t t);
  int idsa_host(IDSA_EVENT * e, char *h);

/* optional comment: describe event to human reading it */
  int idsa_comment(IDSA_EVENT * e, char *m,...);

/* add your own units to an event */
  int idsa_add_string(IDSA_EVENT * e, char *n, char *s);
  int idsa_add_integer(IDSA_EVENT * e, char *n, int i);
  int idsa_add_scan(IDSA_EVENT * e, char *n, unsigned int t, char *s);
  int idsa_add_set(IDSA_EVENT * e, char *n, unsigned int t, void *p);
  int idsa_add_unit(IDSA_EVENT * e, IDSA_UNIT * u);

/* risk / cost defines **************************************************** */

#define IDSA_R_TOTAL       1.00
#define IDSA_R_PARTIAL     0.50
#define IDSA_R_MINOR       0.25
#define IDSA_R_NONE        0.00

#define IDSA_R_UNKNOWN     0.60
#define IDSA_R_RECOVERABLE 0.10

/* log event and get at results ******************************************** */

#define IDSA_L_DENY       0x01	/* event is disallowed */
#define IDSA_L_ALLOW      0x00	/* event is permitted */
#define IDSA_L_OK         IDSA_L_ALLOW
#define IDSA_L_FORWARD    0x02	/* event forwarded to main logger (unused) */
  int idsa_log(IDSA_CONNECTION * c, IDSA_EVENT * e);

/* get at error */
  int idsa_error(IDSA_CONNECTION * c);

/* get reasons for denying / allowing (may return NULL) */
  char *idsa_reason(IDSA_CONNECTION * c);

#ifdef __cplusplus
}

#endif
#endif
