
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <sys/utsname.h>

#include "types.h"
#include "set.h"
#include "rule.h"
#include "table.h"
#include "output.h"
#include "branch.h"

static RULE_BODY *set_trigger(STATE_SET * s, IDSA_EVENT * e);
static void set_result(STATE_SET * c, IDSA_EVENT * e);
static int set_run(STATE_SET * s, IDSA_EVENT * em, IDSA_EVENT * er, JOB * j, RULE_BODY * r);

/****************************************************************************/

STATE_SET *set_new()
{
  STATE_SET *s;
  struct utsname ut;

  s = malloc(sizeof(STATE_SET));
  if (s) {
    s->s_branch = NULL;
    s->s_queue = NULL;

    s->s_rules = NULL;
    s->s_currentrule = NULL;
    s->s_rulecount = 0;

    s->s_output = NULL;
    s->s_outputfail = 0;

    s->s_errno = 0;
    s->s_errbuf[0] = '\0';

    s->s_gid = 0;

    s->s_jobcount = 0;
    s->s_jobmax = IDSAD_STARTJOBS;

    s->s_hostname = strdup(uname(&ut) ? "localhost" : ut.nodename);
    s->s_jobs = malloc(sizeof(JOB) * s->s_jobmax);
    s->s_template = idsa_event_new(0);
    s->s_internal = idsa_event_new(0);
    s->s_message = idsa_event_new(0);
    s->s_reply = idsa_event_new(0);

    if (s->s_hostname && s->s_jobs && s->s_template && s->s_internal && s->s_message && s->s_reply) {
      idsa_message_default(s->s_template, "idsad", "idsa", NULL);
    } else {
      set_free(s);
      s = NULL;
    }
  }
  return s;
}

void set_free(STATE_SET * s)
{
  RULE_BODY *q, *r;
  JOB *j;
  int i;

  /* clear out switches */
  if (s->s_branch) {
    branch_free(s->s_branch);
    s->s_branch = NULL;
  }
  if (s->s_queue) {
    queue_var_free(s->s_queue);
    s->s_queue = NULL;
  }
  /* clear out rules */
  r = s->s_rules;
  while (r) {
    q = r;
    r = r->r_next;
    rule_free(q);
  }
  s->s_rules = NULL;
  s->s_currentrule = NULL;
  s->s_rulecount = 0;

  /* empty output */
  if (s->s_output) {
    output_free(s->s_output);
    s->s_output = NULL;
  }
  /* delete hostname */
  if (s->s_hostname) {
    free(s->s_hostname);
    s->s_hostname = NULL;
  }
  /* close jobs */
  if (s->s_jobs) {
    for (i = 0; i < s->s_jobcount; i++) {
      j = &(s->s_jobs[i]);
      job_end(j);
    }
    free(s->s_jobs);
    s->s_jobs = NULL;
    s->s_jobmax = 0;
    s->s_jobcount = 0;
  }
  if (s->s_template) {
    idsa_event_free(s->s_template);
    s->s_template = NULL;
  }
  if (s->s_internal) {
    idsa_event_free(s->s_internal);
    s->s_internal = NULL;
  }
  if (s->s_message) {
    idsa_event_free(s->s_message);
    s->s_message = NULL;
  }
  if (s->s_reply) {
    idsa_event_free(s->s_reply);
    s->s_reply = NULL;
  }
  free(s);
}

void set_clear(STATE_SET * s)
{
  rule_clear(s->s_rules);
}

static void set_result(STATE_SET * s, IDSA_EVENT * e)
{
  if (s->s_policy & RULE_ACTION_DENY) {
    e->e_result = IDSA_L_DENY;
  } else {
    e->e_result = IDSA_L_ALLOW;
  }
}

int set_drop(STATE_SET * s)
{
  if (s->s_policy & RULE_ACTION_DROP) {
    return 1;
  } else {
    return 0;
  }
}

int set_do(STATE_SET * s, IDSA_EVENT * em, IDSA_EVENT * er, JOB * j)
{
  RULE_BODY *r;

  r = set_trigger(s, em);
  set_result(s, em);

  if (er) {			/* fill in a reply */
    idsa_reply_default(er);
    set_result(s, er);
#ifdef TRACE
    fprintf(stderr, "set_do(): result is %d\n", er->e_result);
#endif
  }
  set_run(s, em, er, j, r);

  return 0;
}

static int set_run(STATE_SET * s, IDSA_EVENT * em, IDSA_EVENT * er, JOB * j, RULE_BODY * r)
{
  RULE_BODY *i;

  /* could be optimized */
  i = s->s_rules;
  while (r != i) {
    if (i->r_match->m_fire == RULE_FIRE_TRUE) {
      rule_run(s, i, em, er, j);
    }
    i = i->r_next;
  }

  return 0;
}

/****************************************************************************/

/****************************************************************************/
/* Does       : fires rules                                                 */
/* Returns    : first rule which does not need to run or NULL if all have   */
/*              to be run                                                   */

static RULE_BODY *set_trigger(STATE_SET * s, IDSA_EVENT * e)
{
  RULE_BODY *r;
  UNION_BRANCH *b;
  RULE_BODY *result;
  int run;

  result = NULL;

  /* default is to allow */
  s->s_policy = RULE_ACTION_ALLOW;

  r = s->s_rules;
  b = s->s_branch;
  if (r && b) {

    branch_trigger(b, e);

    run = 1;
    while (run) {

#ifdef TRACE
      rule_dump(r, stderr);
#endif

      switch (r->r_match->m_fire) {
      case RULE_FIRE_CLEAR:	/* bugger, no rule fired, try next branch */
#ifdef TRACE
	fprintf(stderr, "set_trigger(): rule %d still undecided\n", r->r_number);
#endif
	b = b->b_next;
	if (b) {
	  branch_trigger(b, e);
	} else {		/* should not happen: after evaluating all branches all rules should be decided */
	  run = 0;
	}
	break;
      case RULE_FIRE_TRUE:
#ifdef TRACE
	fprintf(stderr, "set_trigger(): fired rule %d, %p, policy 0x%02x\n", r->r_number, r, r->r_policy);
#endif

	s->s_policy = (s->s_policy | r->r_policy);	/* add up all policies */

	b = branch_reorder(s, b);	/* move this branch one forward */

	if (r->r_policy & RULE_ACTION_CONTINUE) {	/* check if we have to carry on */
	  r = r->r_next;
	  if (r == NULL) {	/* even though continue we run outof rules */
	    run = 0;
	  }
	} else {		/* no continue, only run rules up to r->r_next */
	  result = r->r_next;
	  run = 0;
	}
	break;
      case RULE_FIRE_FALSE:
#ifdef TRACE
	fprintf(stderr, "set_trigger(): rule %d failed\n", r->r_number);
#endif

	r = r->r_next;
	if (r == NULL) {	/* out of rules, quit */
	  run = 0;
	}
	break;
      default:			/* not reached, but in case of failure */
	run = 0;
	break;
      }
    }
  }
  return result;
}
